@@ -219,6 +219,62 @@ def outfile_exists(outfile):
         return(False)
 
 
+def find_random_file(putative_base_file_name,
+                     stringdb_dir = "/usr/local/lib/cell_signals/",
+                     gaurenteed_file_base_file_name="None"):
+    putative_human_file = os.path.join(stringdb_dir,putative_base_file_name)
+    if os.path.isfile(putative_human_file):
+        return(putative_human_file, True)
+    else:
+        found_it = False
+    gaurenteed_file = os.path.join(stringdb_dir,gaurenteed_file_base_file_name)
+    if os.path.isfile(gaurenteed_file):
+        gaurenteed_dir = stringdb_dir
+        found_gaurenteed = True
+    if not os.path.isfile(putative_human_file):
+        print("we couldn't find the file in the supplied string_db dir:")
+        print(stringdb_dir)
+        print("will look around a bit!")
+        ## gather a bunch of possible locations dependeing on if being run as a script or not
+        no_import = False
+        try:
+            from pyminer import pyminer_objs
+        except:
+            print("couldn't import! This must be a script")
+            no_import = True
+        import site, sysconfig
+        dirs_to_try = []
+        if not no_import:
+            dirs_to_try.append(os.path.dirname(pyminer_objs.__file__))
+        if "getsitepackages" in dir(site):
+            site_packs = site.getsitepackages()
+            for i in range(len(site_packs)):
+                site_packs[i]=os.path.join(site_packs[i],"pyminer")
+            dirs_to_try += site_packs
+        dirs_to_try.append(os.path.join(sysconfig.get_paths()["purelib"],"pyminer"))
+        ## now try the locations
+        for test_dir in dirs_to_try:
+            putative_relative_human_file = os.path.join(test_dir,putative_base_file_name)
+            gaurenteed_file = os.path.join(test_dir,gaurenteed_file_base_file_name)
+            print(putative_relative_human_file)
+            print(gaurenteed_file)
+            if os.path.isfile(putative_relative_human_file):
+                print("\n\nFound it!\n", "using this as the stringdb_dir:",test_dir)
+                stringdb_dir = test_dir
+                found_it = True
+                return(putative_relative_human_file, True)
+            if os.path.isfile(gaurenteed_file):
+                gaurenteed_dir = test_dir
+                found_gaurenteed = True
+        if not found_it and found_gaurenteed:
+            print("\n\n\n\nWARNING! couldn't find the right files in the stringdb_dir:",stringdb_dir)
+            print("but we did find the gaurenteed file & will convert from there")
+            return(os.path.join(gaurenteed_dir, gaurenteed_file_base_file_name), False)
+        if not found_it and not found_gaurenteed:
+            return(None, False)
+        
+                
+
 
 
 # def get_sample_k_lists(group_numeric_vector, total_groups = None):
@@ -425,11 +481,23 @@ def get_num_cols_from_dict_lists(dict_list):
     first = dict_list[0][first_key]
     if type(first) == list:
         col_dims = len(first)
+    elif type(first) == np.ndarray:
+        col_dims=first.shape[0]
     else:
         col_dims = 1
     return(col_dims)
 
 
+def ray_dict_of_array_slices_to_array(dict_list):
+    row_dims = get_num_rows_from_dict_lists(dict_list)
+    col_dims = get_num_cols_from_dict_lists(dict_list)
+    out_array = np.zeros((row_dims, col_dims))
+    for temp_dict in dict_list:
+        for idx, value in temp_dict.items():
+            out_array[idx,:]=value
+    return(out_array)
+
+
 def ray_dicts_to_array(dict_list, csc = False):
     row_dims = get_num_rows_from_dict_lists(dict_list)
     col_dims = get_num_cols_from_dict_lists(dict_list)
