@@ -4,6 +4,7 @@ import numpy as np
 import random
 import scipy
 from subprocess import Popen
+from sklearn.decomposition import PCA
 from time import sleep
 from scipy.sparse import csr_matrix, dok_matrix
 from scipy.stats import spearmanr
@@ -150,13 +151,33 @@ parser.add_argument("-log_odds","-lod",
     default = False, 
     help="if you only want to use proportionality instead of Spearman")
 
+parser.add_argument(
+    "-skip_spearman",
+    help="If we're going to skip the spearman correlations and just do the euclidean distances",
+    action = "store_true",
+    default = False)
+
+parser.add_argument(
+    "-pca_reduce",
+    help="If we're going to perform a pca dimension reduction prior to spearman/euclidean dist calcs",
+    action = "store_true",
+    default = False)
+
+parser.add_argument(
+    "-n_pcs",
+    help="The number of principle components to keep for a PCA dim reduction. Ignored if pca_reduce isn't used",
+    type = int,
+    default = 30)
+
 args = parser.parse_args()
 
 
 np.random.seed(args.rand_seed)
 random.seed(args.rand_seed)
 
-
+skip_spearman=args.skip_spearman
+pca_reduce=args.pca_reduce
+n_pcs=args.n_pcs
 
 ## constants
 FPR = 1000 ## 1 in 1000 false positive rate for negative correlation clustering
@@ -479,255 +500,277 @@ def get_rho_cutoff_vect(r, neg_control_sample_idxs, gene_percent_express, out_di
     sys.exit()
     return()
 
+################################################
+################################################
+if pca_reduce:
+    ## samples in rows, PCs in cols
+    # for clustering, it has already been transposed from typical genes=rows, cells=cols format, so no need to worry
+    print("doing PCA reduction:",str(n_pcs)," dims")
+    pca=PCA(n_components=n_pcs)
+    infile = pca.fit_transform(infile)
+    print(infile.shape)
 
-##########
-## get the null
-log_odds_correction = 0
-if do_sum_neg or args.rho_cutoff==None or args.log_odds:
-    ## run the bootstrap negative control for finding sum negative relationships
-    ## first get a bin_size random set of variables to do the negative control on
-    neg_control_sample_idxs = np.arange(len(IDlist))
-    print(neg_control_sample_idxs)
-    ## remove any genes that are zero in all samples
-    non_zero_idxs = np.where(np.sum(infile,axis=1)>0)[0]
-    neg_control_sample_idxs = neg_control_sample_idxs[non_zero_idxs]
-    np.random.shuffle(neg_control_sample_idxs)
-    #print(np.shape(neg_control_sample_idxs))
-    neg_ctrl_bin_size = min([bin_size, np.shape(neg_control_sample_idxs)[0]])
-    print("bin_size",neg_ctrl_bin_size)
-    neg_control_sample_idxs = neg_control_sample_idxs[:neg_ctrl_bin_size]
-    ##############################################################
-    ## 11/14/2020
-    ## add lowest & highest % expression to make sure we cover the whole range
-    ## & don't cause an error with lowess later
-    if not single_cutoff:
-        percent_express_sorted_idxs = np.argsort(gene_percent_express)
-        highest_percent_idx = percent_express_sorted_idxs[0]
-        lowest_percent_idx = percent_express_sorted_idxs[percent_express_sorted_idxs.shape[0]-1]
-        if highest_percent_idx not in neg_control_sample_idxs:
-            neg_control_sample_idxs[0] = highest_percent_idx
-        if lowest_percent_idx not in neg_control_sample_idxs:
-            neg_control_sample_idxs[-1] = lowest_percent_idx
-    ##############################################################
-    neg_control_sample_idxs = neg_control_sample_idxs.tolist()
-    neg_control_sample_idxs.sort()
-    #print(np.shape(neg_control_sample_idxs))
 
-    ## subset these IDs
-    neg_control_subset_mat = np.array(infile[neg_control_sample_idxs,:])
-    print('making negative control bootstrap shuffled matrix',np.shape(neg_control_subset_mat))
 
-    ## go through each row and shuffle them within variables
-    for i in range(0,np.shape(neg_control_subset_mat)[0]):
-        temp_row_shuffle_order = np.arange(np.shape(neg_control_subset_mat)[1])
-        np.random.shuffle(temp_row_shuffle_order)
-        temp_row_shuffle_order = temp_row_shuffle_order.tolist()
-        neg_control_subset_mat[i,:] = neg_control_subset_mat[i,temp_row_shuffle_order]
 
-    ## get the spearman_rhos for the shuffled matrix
-    print("neg_control_subset_mat shape:",np.shape(neg_control_subset_mat)[0])
-    if np.shape(neg_control_subset_mat)[0]<2:
-        ## this is likely a bad input, but handle it just in case it comes up
-        r=np.zeros((neg_ctrl_bin_size,neg_ctrl_bin_size))
-        linear_rho = np.zeros(neg_ctrl_bin_size)
-        neg_cutoff=0
-        FPR=0
-        rho_cutoff = 0
-        args.rho_cutoff = 0
-    else:
-        r=no_p_spear(neg_control_subset_mat, neg_control_subset_mat, axis = 1, prop=args.prop, log_odds = args.log_odds)#, log_odds_correction = log_odds_correction)
-        linear_rho = get_linear_triangle(r[:neg_ctrl_bin_size,:neg_ctrl_bin_size], neg_ctrl_bin_size)
-        #########################################
-        ## THIS IS WHERE WE SET THE LOG-ODDS BIAS CORRECTION. It's just the median of the null distribution to center it & future comparisons
-        if args.log_odds:
-            log_odds_correction = np.median(linear_rho)
-            print("\n\n\nlog_odds_correction:", log_odds_correction,"\n\n\n\n")
-            linear_rho -= log_odds_correction
-        else:
-            log_odds_correction = 0
-        #########################################
-        print("min of negative control is now:", np.min(linear_rho))
-        print("median of negative control is now:", np.median(linear_rho))
-        print("max of negative control is now:", np.max(linear_rho))
-        print("rho shape",np.shape(r))
-        
-        if single_cutoff:
-            print("linear_rho.shape:",linear_rho.shape)
-            neg_idxs = np.where(linear_rho<0)[0]
-            print("np.where(linear_rho<0)",np.where(linear_rho<0))
-            pos_idxs = np.where(linear_rho>0)[0]
-            negative_control_negative_rho_vect = linear_rho[neg_idxs]
-            negative_control_positive_rho_vect = linear_rho[pos_idxs]
-            #calculate_mad(negative_control_negative_rho_vect)
 
-            ## plot the average Rho per negative correlation
-            #print(negative_control_negative_rho_vect)
-            
-            if not args.transpose:
-                neg_cor_density_x, neg_cor_density_y = get_density(negative_control_negative_rho_vect)
-                pos_cor_density_x, pos_cor_density_y = get_density(negative_control_positive_rho_vect)
-                ## this is only for gene-gene correlations, not sample-sample
-                print(args.adj_list_out)
-                out_dir = os.path.dirname(args.adj_list_out)
-                sum_neg_plot = os.path.join(out_dir,'boostrap_neg_cor_rhos.png')
-                print("plotting boostrap_neg_cor_rhos.png")
-                plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot, label="density of negative correlation null distribution")
-                pos_rho_plot = os.path.join(out_dir,'boostrap_pos_cor_rhos.png')
-                print("plotting boostrap_pos_cor_rhos.png")
-                plot_one_density(pos_cor_density_x, pos_cor_density_y,pos_rho_plot, label="density of positive correlation null distribution")
+################################################
+################################################
 
-                ## plot the distribution of all bootstrap negative control Rhos
-                neg_cor_density_x, neg_cor_density_y = get_density(linear_rho)
-                sum_neg_plot = os.path.join(out_dir,'boostrap_cor_rhos.png')
-                print("plotting boostrap_cor_rhos.png")
-                plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot,label="density of all correlation null distribution")
 
-            print("\n\ncalculating negative correlation FPR cutoff from null distribution:")
-            #neg_cutoff = get_FPR_cutoff(negative_control_negative_rho_vect,FPR)
-            #neg_cutoff, FPR = get_Z_cutoff(negative_control_negative_rho_vect, z=3)
-            if args.log_odds:
-                neg_cutoff, FPR = get_empiric_FPR(negative_control_negative_rho_vect)
-            else:
-                neg_cutoff, FPR = get_empiric_FPR(negative_control_negative_rho_vect)
-            print('\nEmpiric FPR for anti-corr:',FPR)
-            print('\nEmpiric neg_cutoff for anti-corr:',neg_cutoff)
-            print("\n\ncalculating adj list FPR cutoffs from null distribution:")
-            #pos_cutoff = get_FPR_cutoff(negative_control_positive_rho_vect,pos_FPR, positive = True)
-            if args.log_odds:
-                pos_z_cut = 3.0
-                pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=pos_z_cut, positive = True, cap=None)
-                neg_cutoff_adj_list, dummy = get_Z_cutoff(negative_control_negative_rho_vect, z=pos_z_cut, positive = False, cap=None)
-            else:
-                pos_z_cut = 5.5
-                pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=pos_z_cut, positive = True)
-                neg_cutoff_adj_list, dummy = get_Z_cutoff(negative_control_negative_rho_vect, z=pos_z_cut, positive = False)
-            rho_cutoff = pos_cutoff
-            args.rho_cutoff = pos_cutoff
-            #sys.exit()
-        else:
-            ##############################################################
-            ## this means that we just have a dynamic negative rho cutoff as determined by gene expression sparsity
-            rho_cutoff_vect, FPR_vect = get_rho_cutoff_vect(r, neg_control_sample_idxs, gene_non_zero_mean, os.path.dirname(args.adj_list_out))
-    ##############################################################################################################################################
-##############################################################################################################################################
-##############################################################################################################################################
-## Do all of the comparisons
-
-start = time.time()
-if total_vars==bins[1] and args.hdf5_out:
-    print("total num vars is equal to the number of bins, so we'll do it in one go")
-    r=no_p_spear(infile[:,:], axis = 1, prop=args.prop, log_odds = args.log_odds, log_odds_correction = log_odds_correction)
-    print(r.shape)
-    print(spear_out_hdf5.shape)
-    spear_out_hdf5[:,:]=r[:,:]
-else:
-    print("finding block...")
-    for i in range(0,(len(bins)-1)):
-        temp_first = infile[bins[i]:bins[i+1],]
+if not skip_spearman:
+    ##########
+    ## get the null
+    log_odds_correction = 0
+    if do_sum_neg or args.rho_cutoff==None or args.log_odds:
+        ## run the bootstrap negative control for finding sum negative relationships
+        ## first get a bin_size random set of variables to do the negative control on
+        neg_control_sample_idxs = np.arange(len(IDlist))
+        print(neg_control_sample_idxs)
+        ## remove any genes that are zero in all samples
+        non_zero_idxs = np.where(np.sum(infile,axis=1)>0)[0]
+        neg_control_sample_idxs = neg_control_sample_idxs[non_zero_idxs]
+        np.random.shuffle(neg_control_sample_idxs)
+        #print(np.shape(neg_control_sample_idxs))
+        neg_ctrl_bin_size = min([bin_size, np.shape(neg_control_sample_idxs)[0]])
+        print("bin_size",neg_ctrl_bin_size)
+        neg_control_sample_idxs = neg_control_sample_idxs[:neg_ctrl_bin_size]
+        ##############################################################
+        ## 11/14/2020
+        ## add lowest & highest % expression to make sure we cover the whole range
+        ## & don't cause an error with lowess later
         if not single_cutoff:
-            gene_num_zero[bins[i]:bins[i+1]] = np.sum(temp_first==0,axis=1)
-            gene_non_zero_mean[bins[i]:bins[i+1]] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[i]:bins[i+1]])
-        for j in range(i,(len(bins)-1)):
-            if (i!=j) or (len(bins) == 2):#True:
-                out_rho_dict = rho_dict_dir+"/rho_"+str(bins[i])+"_to_"+str(bins[i+1])+"_vs_"+str(bins[j])+"_to_"+str(bins[j+1])+".pkl"
+            percent_express_sorted_idxs = np.argsort(gene_percent_express)
+            highest_percent_idx = percent_express_sorted_idxs[0]
+            lowest_percent_idx = percent_express_sorted_idxs[percent_express_sorted_idxs.shape[0]-1]
+            if highest_percent_idx not in neg_control_sample_idxs:
+                neg_control_sample_idxs[0] = highest_percent_idx
+            if lowest_percent_idx not in neg_control_sample_idxs:
+                neg_control_sample_idxs[-1] = lowest_percent_idx
+        ##############################################################
+        neg_control_sample_idxs = neg_control_sample_idxs.tolist()
+        neg_control_sample_idxs.sort()
+        #print(np.shape(neg_control_sample_idxs))
 
-                ## if we're writing to an hdf5 file, check if the values are already entered
-                if args.hdf5_out:
-                    #do_top_left = np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]]==0)
-                    #do_top_right = np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]]==0)
-                    #do_bottom_left = np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1]]==0)
-                    #do_bottom_right = np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]]==0)
-                    do_top_left = np.all(spear_out_hdf5[bins[i],bins[i]:bins[i+1]]==0)
-                    do_top_right = np.all(spear_out_hdf5[bins[i],bins[j]:bins[j+1]]==0)
-                    do_bottom_left = np.all(spear_out_hdf5[bins[j],bins[i]:bins[i+1]]==0)
-                    do_bottom_right = np.all(spear_out_hdf5[bins[j],bins[j]:bins[j+1]]==0)
-                    do_one = (do_top_left or do_top_right or do_bottom_left or do_bottom_right)
-                    print(do_top_left , do_top_right , do_bottom_left , do_bottom_right)
-                    #print(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]])
-                else:
-                    do_one = False
+        ## subset these IDs
+        neg_control_subset_mat = np.array(infile[neg_control_sample_idxs,:])
+        print('making negative control bootstrap shuffled matrix',np.shape(neg_control_subset_mat))
+
+        ## go through each row and shuffle them within variables
+        for i in range(0,np.shape(neg_control_subset_mat)[0]):
+            temp_row_shuffle_order = np.arange(np.shape(neg_control_subset_mat)[1])
+            np.random.shuffle(temp_row_shuffle_order)
+            temp_row_shuffle_order = temp_row_shuffle_order.tolist()
+            neg_control_subset_mat[i,:] = neg_control_subset_mat[i,temp_row_shuffle_order]
+
+        ## get the spearman_rhos for the shuffled matrix
+        print("neg_control_subset_mat shape:",np.shape(neg_control_subset_mat)[0])
+        if np.shape(neg_control_subset_mat)[0]<2:
+            ## this is likely a bad input, but handle it just in case it comes up
+            r=np.zeros((neg_ctrl_bin_size,neg_ctrl_bin_size))
+            linear_rho = np.zeros(neg_ctrl_bin_size)
+            neg_cutoff=0
+            FPR=0
+            rho_cutoff = 0
+            args.rho_cutoff = 0
+        else:
+            r=no_p_spear(neg_control_subset_mat, neg_control_subset_mat, axis = 1, prop=args.prop, log_odds = args.log_odds)#, log_odds_correction = log_odds_correction)
+            linear_rho = get_linear_triangle(r[:neg_ctrl_bin_size,:neg_ctrl_bin_size], neg_ctrl_bin_size)
+            #########################################
+            ## THIS IS WHERE WE SET THE LOG-ODDS BIAS CORRECTION. It's just the median of the null distribution to center it & future comparisons
+            if args.log_odds:
+                log_odds_correction = np.median(linear_rho)
+                print("\n\n\nlog_odds_correction:", log_odds_correction,"\n\n\n\n")
+                linear_rho -= log_odds_correction
+            else:
+                log_odds_correction = 0
+            #########################################
+            print("min of negative control is now:", np.min(linear_rho))
+            print("median of negative control is now:", np.median(linear_rho))
+            print("max of negative control is now:", np.max(linear_rho))
+            print("rho shape",np.shape(r))
+            
+            if single_cutoff:
+                print("linear_rho.shape:",linear_rho.shape)
+                neg_idxs = np.where(linear_rho<0)[0]
+                print("np.where(linear_rho<0)",np.where(linear_rho<0))
+                pos_idxs = np.where(linear_rho>0)[0]
+                negative_control_negative_rho_vect = linear_rho[neg_idxs]
+                negative_control_positive_rho_vect = linear_rho[pos_idxs]
+                #calculate_mad(negative_control_negative_rho_vect)
+
+                ## plot the average Rho per negative correlation
+                #print(negative_control_negative_rho_vect)
                 
-                #print((not os.path.exists(out_rho_dict) and not args.hdf5_out) , (args.hdf5_out and do_one))
-                #sys.exit()
-                if (not os.path.exists(out_rho_dict) and not args.hdf5_out) or (args.hdf5_out and do_one):
-                    ## find the spearman correlations
+                if not args.transpose:
+                    neg_cor_density_x, neg_cor_density_y = get_density(negative_control_negative_rho_vect)
+                    pos_cor_density_x, pos_cor_density_y = get_density(negative_control_positive_rho_vect)
+                    ## this is only for gene-gene correlations, not sample-sample
+                    print(args.adj_list_out)
+                    out_dir = os.path.dirname(args.adj_list_out)
+                    sum_neg_plot = os.path.join(out_dir,'boostrap_neg_cor_rhos.png')
+                    print("plotting boostrap_neg_cor_rhos.png")
+                    plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot, label="density of negative correlation null distribution")
+                    pos_rho_plot = os.path.join(out_dir,'boostrap_pos_cor_rhos.png')
+                    print("plotting boostrap_pos_cor_rhos.png")
+                    plot_one_density(pos_cor_density_x, pos_cor_density_y,pos_rho_plot, label="density of positive correlation null distribution")
 
-                    print('working on',bins[i],bins[i+1],'vs',bins[j],bins[j+1], "prop:",args.prop, "log_odds:",args.log_odds,log_odds_correction)
-                    r=no_p_spear(temp_first,infile[bins[j]:bins[j+1],:], axis = 1, prop=args.prop, log_odds = args.log_odds, log_odds_correction = log_odds_correction)
-                    
-                    ## if we're not making the spearman correlation matrix into an hdf5 file - save the dictionary:
-                    if not args.hdf5_out:
-                        save_dict(r,out_rho_dict)
+                    ## plot the distribution of all bootstrap negative control Rhos
+                    neg_cor_density_x, neg_cor_density_y = get_density(linear_rho)
+                    sum_neg_plot = os.path.join(out_dir,'boostrap_cor_rhos.png')
+                    print("plotting boostrap_cor_rhos.png")
+                    plot_one_density(neg_cor_density_x, neg_cor_density_y, sum_neg_plot,label="density of all correlation null distribution")
 
-                    ## if we are making the spearman correlation matrix an hdf5 file, we've got some work to do:
-                    else:
-                        #print(np.shape(r))
-                        if np.shape(r)[0] == 2*bin_size:
-                            #print('is symmetric:',r[bin_size:,bin_size:] == r[:bin_size,:bin_size])
-                            if len(bins)==2:#False#np.shape(r)[0]<=bin_size:
-                                ## there is a special case, where the number of variables is less than the
-                                ## bin size. In this case, the returned matrix will be symmetric in all 4
-                                ## quadrants, so we'll reduce it down to just the one
-                                r = r[bin_size:,bin_size:]
-                        #print(np.shape(r))
-
-
-                        ## because h5py's fancy indexing doesn't work very well with slicing in 2D...
-
-                        ## first set the top left corner bins
-                        if np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]]==0):
-                            print("setting top left")
-                            # temp_r = np.array(r[:bin_size,:bin_size],dtype=np.float16)
-                            # print(temp_r.shape)
-                            print(bins[i],bins[i+1])
-                            print()
-                            if spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]].shape[0]-1==r[:bin_size,:bin_size].shape[0]:
-                                spear_out_hdf5[bins[i]:bins[i+1]-1,bins[i]:bins[i+1]-1] = np.array(r[:bin_size,:bin_size],dtype=np.float16)
-                            else:
-                                spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]] = np.array(r[:bin_size,:bin_size],dtype=np.float16)
-                        
-                        ## top right
-                        if len(bins)!=2:
-                            try:
-                                if np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]]==0):
-                                    print("setting top right")
-                                    spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]] = np.array(r[:bin_size,bin_size:],dtype=np.float16)
-                            except:
-                                print(sys.exit('error setting the top right'))
-                                pass
-                        
-                        ## bottom left
-                        if len(bins)!=2:
-                            try:
-                                if np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1]]==0):
-                                    print('setting bottom left')
-                                    spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1],] = np.array(r[bin_size:,:bin_size],dtype=np.float16)
-                            except:
-                                print(sys.exit('error setting the bottom left'))
-                                pass
-                        
-                        ## bottom right
-                        if len(bins)!=2:
-                            try:
-                                if np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]]==0):
-                                    print('setting bottom right')
-                                    spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]] = np.array(r[bin_size:,bin_size:],dtype=np.float16)
-                            except:
-                                print(sys.exit('error setting the bottom right'))
-                                pass
+                print("\n\ncalculating negative correlation FPR cutoff from null distribution:")
+                #neg_cutoff = get_FPR_cutoff(negative_control_negative_rho_vect,FPR)
+                #neg_cutoff, FPR = get_Z_cutoff(negative_control_negative_rho_vect, z=3)
+                if args.log_odds:
+                    neg_cutoff, FPR = get_empiric_FPR(negative_control_negative_rho_vect)
                 else:
-                    print('already finished block',bins[i],bins[i+1],'vs',bins[j],bins[j+1])
+                    neg_cutoff, FPR = get_empiric_FPR(negative_control_negative_rho_vect)
+                print('\nEmpiric FPR for anti-corr:',FPR)
+                print('\nEmpiric neg_cutoff for anti-corr:',neg_cutoff)
+                print("\n\ncalculating adj list FPR cutoffs from null distribution:")
+                #pos_cutoff = get_FPR_cutoff(negative_control_positive_rho_vect,pos_FPR, positive = True)
+                if args.log_odds:
+                    pos_z_cut = 3.0
+                    pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=pos_z_cut, positive = True, cap=None)
+                    neg_cutoff_adj_list, dummy = get_Z_cutoff(negative_control_negative_rho_vect, z=pos_z_cut, positive = False, cap=None)
+                else:
+                    pos_z_cut = 5.5
+                    pos_cutoff = get_Z_cutoff(negative_control_positive_rho_vect, z=pos_z_cut, positive = True)
+                    neg_cutoff_adj_list, dummy = get_Z_cutoff(negative_control_negative_rho_vect, z=pos_z_cut, positive = False)
+                rho_cutoff = pos_cutoff
+                args.rho_cutoff = pos_cutoff
+                #sys.exit()
+            else:
+                ##############################################################
+                ## this means that we just have a dynamic negative rho cutoff as determined by gene expression sparsity
+                rho_cutoff_vect, FPR_vect = get_rho_cutoff_vect(r, neg_control_sample_idxs, gene_non_zero_mean, os.path.dirname(args.adj_list_out))
+        ##############################################################################################################################################
+    ##############################################################################################################################################
+    ##############################################################################################################################################
+    ## Do all of the comparisons
 
-end = time.time()
+    start = time.time()
+    if total_vars==bins[1] and args.hdf5_out:
+        print("total num vars is equal to the number of bins, so we'll do it in one go")
+        r=no_p_spear(infile[:,:], axis = 1, prop=args.prop, log_odds = args.log_odds, log_odds_correction = log_odds_correction)
+        print(r.shape)
+        print(spear_out_hdf5.shape)
+        spear_out_hdf5[:,:]=r[:,:]
+    else:
+        print("finding block...")
+        for i in range(0,(len(bins)-1)):
+            temp_first = infile[bins[i]:bins[i+1],]
+            if not single_cutoff:
+                gene_num_zero[bins[i]:bins[i+1]] = np.sum(temp_first==0,axis=1)
+                gene_non_zero_mean[bins[i]:bins[i+1]] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[i]:bins[i+1]])
+            for j in range(i,(len(bins)-1)):
+                if (i!=j) or (len(bins) == 2):#True:
+                    out_rho_dict = rho_dict_dir+"/rho_"+str(bins[i])+"_to_"+str(bins[i+1])+"_vs_"+str(bins[j])+"_to_"+str(bins[j+1])+".pkl"
+
+                    ## if we're writing to an hdf5 file, check if the values are already entered
+                    if args.hdf5_out:
+                        #do_top_left = np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]]==0)
+                        #do_top_right = np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]]==0)
+                        #do_bottom_left = np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1]]==0)
+                        #do_bottom_right = np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]]==0)
+                        do_top_left = np.all(spear_out_hdf5[bins[i],bins[i]:bins[i+1]]==0)
+                        do_top_right = np.all(spear_out_hdf5[bins[i],bins[j]:bins[j+1]]==0)
+                        do_bottom_left = np.all(spear_out_hdf5[bins[j],bins[i]:bins[i+1]]==0)
+                        do_bottom_right = np.all(spear_out_hdf5[bins[j],bins[j]:bins[j+1]]==0)
+                        do_one = (do_top_left or do_top_right or do_bottom_left or do_bottom_right)
+                        print(do_top_left , do_top_right , do_bottom_left , do_bottom_right)
+                        #print(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]])
+                    else:
+                        do_one = False
+                    
+                    #print((not os.path.exists(out_rho_dict) and not args.hdf5_out) , (args.hdf5_out and do_one))
+                    #sys.exit()
+                    if (not os.path.exists(out_rho_dict) and not args.hdf5_out) or (args.hdf5_out and do_one):
+                        ## find the spearman correlations
+
+                        print('working on',bins[i],bins[i+1],'vs',bins[j],bins[j+1], "prop:",args.prop, "log_odds:",args.log_odds,log_odds_correction)
+                        r=no_p_spear(temp_first,infile[bins[j]:bins[j+1],:], axis = 1, prop=args.prop, log_odds = args.log_odds, log_odds_correction = log_odds_correction)
+                        
+                        ## if we're not making the spearman correlation matrix into an hdf5 file - save the dictionary:
+                        if not args.hdf5_out:
+                            save_dict(r,out_rho_dict)
+
+                        ## if we are making the spearman correlation matrix an hdf5 file, we've got some work to do:
+                        else:
+                            #print(np.shape(r))
+                            if np.shape(r)[0] == 2*bin_size:
+                                #print('is symmetric:',r[bin_size:,bin_size:] == r[:bin_size,:bin_size])
+                                if len(bins)==2:#False#np.shape(r)[0]<=bin_size:
+                                    ## there is a special case, where the number of variables is less than the
+                                    ## bin size. In this case, the returned matrix will be symmetric in all 4
+                                    ## quadrants, so we'll reduce it down to just the one
+                                    r = r[bin_size:,bin_size:]
+                            #print(np.shape(r))
+
+
+                            ## because h5py's fancy indexing doesn't work very well with slicing in 2D...
+
+                            ## first set the top left corner bins
+                            if np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]]==0):
+                                print("setting top left")
+                                # temp_r = np.array(r[:bin_size,:bin_size],dtype=np.float16)
+                                # print(temp_r.shape)
+                                print(bins[i],bins[i+1])
+                                print()
+                                if spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]].shape[0]-1==r[:bin_size,:bin_size].shape[0]:
+                                    spear_out_hdf5[bins[i]:bins[i+1]-1,bins[i]:bins[i+1]-1] = np.array(r[:bin_size,:bin_size],dtype=np.float16)
+                                else:
+                                    spear_out_hdf5[bins[i]:bins[i+1],bins[i]:bins[i+1]] = np.array(r[:bin_size,:bin_size],dtype=np.float16)
+                            
+                            ## top right
+                            if len(bins)!=2:
+                                try:
+                                    if np.all(spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]]==0):
+                                        print("setting top right")
+                                        spear_out_hdf5[bins[i]:bins[i+1],bins[j]:bins[j+1]] = np.array(r[:bin_size,bin_size:],dtype=np.float16)
+                                except:
+                                    print(sys.exit('error setting the top right'))
+                                    pass
+                            
+                            ## bottom left
+                            if len(bins)!=2:
+                                try:
+                                    if np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1]]==0):
+                                        print('setting bottom left')
+                                        spear_out_hdf5[bins[j]:bins[j+1],bins[i]:bins[i+1],] = np.array(r[bin_size:,:bin_size],dtype=np.float16)
+                                except:
+                                    print(sys.exit('error setting the bottom left'))
+                                    pass
+                            
+                            ## bottom right
+                            if len(bins)!=2:
+                                try:
+                                    if np.all(spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]]==0):
+                                        print('setting bottom right')
+                                        spear_out_hdf5[bins[j]:bins[j+1],bins[j]:bins[j+1]] = np.array(r[bin_size:,bin_size:],dtype=np.float16)
+                                except:
+                                    print(sys.exit('error setting the bottom right'))
+                                    pass
+                    else:
+                        print('already finished block',bins[i],bins[i+1],'vs',bins[j],bins[j+1])
+    end = time.time()
+    ######################
+    ## finish cateloguing the number of zeros
+    if not single_cutoff:
+        temp_first = infile[bins[-1]:,]
+        gene_num_zero[bins[-1]:] = np.sum(temp_first==0,axis=1)
+        gene_non_zero_mean[bins[-1]:] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[-1]:])
+        gene_percent_express = (num_cells - gene_num_zero)/num_cells
+    ######################
+else:
+    print("skipping the Spearman correlations & will continue on (to Euc if specified)")
+    spear_out_hdf5=infile
+    start = time.time()
+    end = time.time()
 
-######################
-## finish cateloguing the number of zeros
-if not single_cutoff:
-    temp_first = infile[bins[-1]:,]
-    gene_num_zero[bins[-1]:] = np.sum(temp_first==0,axis=1)
-    gene_non_zero_mean[bins[-1]:] = np.sum(temp_first,axis=1) / (num_cells - gene_num_zero[bins[-1]:])
-    gene_percent_express = (num_cells - gene_num_zero)/num_cells
-######################
 
 
 ##############################################################################################################################################
@@ -813,13 +856,18 @@ if args.euclidean_dist:
 
 if args.hdf5_out:
     ## close the hdf5 file
-    spear_f.close()
+    try:
+        spear_f.close()
+    except:
+        print("couldn't close 'spear_f' probably because we skipped it")
 
 
 if args.time:
     print((end-start)/60,'minutes spent finding correlations')
     sys.exit()
 
+if skip_spearman:
+    sys.exit()
 
 ###############################################################
 
