@@ -249,12 +249,20 @@ else:
     if args.graph != None:
         ## read in the input graph
         in_graph = import_dict(args.graph)
-        G = in_graph['graph']
+        print(type(in_graph))
+        if type(in_graph)==dict:
+            G = in_graph['graph']
+        else:
+            G = in_graph
         ID_list = G.nodes()
         if 'positions' in in_graph:
             pos = in_graph['positions']
             num_node = number_of_nodes(G)
             num_edge = number_of_edges(G)
+        if 'pos' in in_graph:
+            pos = in_graph['positions']
+            num_node = number_of_nodes(G)
+            num_edge = number_of_edges(G)
     else:
 		## read in the IDs for setting up the layout of the adj matrix
         if args.id_list == None:
