@@ -57,7 +57,7 @@ def do_the_clustering(temp_dir,
         print(temp_dir, infile, do_h5_py, ID_list)
         if not os.path.isdir(temp_dir+'/sample_clustering_and_summary'):
             process_dir(temp_dir+'/sample_clustering_and_summary')
-        dispersion_str = 'pyminer_get_dispersion.py -i '+infile
+        dispersion_str = 'python3 -m pyminer.pyminer_get_dispersion -i '+infile
         dispersion_str += ' -out_dir '+temp_dir+'/sample_clustering_and_summary/dispersion/'
         if do_h5_py:
             dispersion_str += ' -hdf5 '
@@ -65,7 +65,7 @@ def do_the_clustering(temp_dir,
         all_calls.append(dispersion_str)
         cmd(dispersion_str)
 
-    cluster_string = 'clustering.py -i '+infile+' -species '+species
+    cluster_string = 'python3 -m pyminer.clustering -i '+infile+' -species '+species
     if sc_clust:
         cluster_string += ' -sc_clust '
         cluster_string += ' -neg_cor_count_clust '+str(temp_dir+'/sum_neg_cor.txt')
@@ -251,7 +251,7 @@ def pyminer_analysis(infile=None,
     ################################################
     ## get gene annotations
     annotation_prefix = os.path.join(temp,'annotations')
-    annotation_call = 'pyminer_gprofiler_converter.py -i '+id_file_name
+    annotation_call = 'python3 -m pyminer.pyminer_gprofiler_converter -i '+id_file_name
     annotation_call += ' -o '+annotation_prefix
     annotation_call += ' -s '+species
     annotation_call += ' -annotations'
@@ -260,7 +260,7 @@ def pyminer_analysis(infile=None,
     ## get the human orthologues if we don't have human ids
     if species != 'hsapiens':
         orthologue_prefix = os.path.join(temp,'human_orthologues')
-        convert_call = 'pyminer_gprofiler_converter.py -i '+id_file_name
+        convert_call = 'python3 -m pyminer.pyminer_gprofiler_converter -i '+id_file_name
         convert_call += ' -o '+orthologue_prefix
         convert_call += ' -s '+species
         cmd(convert_call)
@@ -273,7 +273,7 @@ def pyminer_analysis(infile=None,
             sys.exit('please use a rho cutoff between zero and one. We recommend not using this argument at all to allow for the built in False Positive Rate (FPR) algorithm to take over. If using scRNAseq, try something in the range of 0.25-0.35. For bulk RNAseq, try something closer to 0.70 - 0.90.')
 
     out_adj = os.path.join(temp,'adj_list_rho.tsv')
-    adj_call_string = 'mat_to_adj_list.py -i '+infile+' -o '+out_adj+' -id_list '+os.path.join(temp,'ID_list.txt ')+' -num_pos_cor_cutoff '+str(num_pos_cor_cutoff)
+    adj_call_string = 'python3 -m pyminer.mat_to_adj_list -i '+infile+' -o '+out_adj+' -id_list '+os.path.join(temp,'ID_list.txt ')+' -num_pos_cor_cutoff '+str(num_pos_cor_cutoff)
     if rho_cutoff!=None:
         adj_call_string+=' -rho '+str(rho_cutoff)
     if do_h5_py:
@@ -472,7 +472,7 @@ def pyminer_analysis(infile=None,
     ################################################
     ## : enrichment & significance
     #print(infile, manual_sample_groups_file, FDR_cutoff, zscore_cutoff, species, ID_list, columns)
-    stats_call = 'get_stats.py -i '+infile+' -sample_groups '+manual_sample_groups_file+' -FDR '+str(FDR_cutoff)+' -Zscore '+str(zscore_cutoff)
+    stats_call = 'python3 -m pyminer.get_stats -i '+infile+' -sample_groups '+manual_sample_groups_file+' -FDR '+str(FDR_cutoff)+' -Zscore '+str(zscore_cutoff)
     if species != None:
         stats_call += str(' -s '+species)
     if do_h5_py:
@@ -501,7 +501,7 @@ def pyminer_analysis(infile=None,
     ################################################
     ## combine the gprofiler results
     if os.path.isdir(os.path.join(temp,"sample_clustering_and_summary/significance/gprofiler")):
-        combine_gprofiler_str = "combine_gprofiler_results.py -i "+temp+"/sample_clustering_and_summary/significance/gprofiler/"
+        combine_gprofiler_str = "python3 -m pyminer.combine_gprofiler_results -i "+temp+"/sample_clustering_and_summary/significance/gprofiler/"
         combine_gprofiler_str +=" -o "+os.path.join(temp,"sample_clustering_and_summary/significance/")
         all_calls.append(combine_gprofiler_str)
         cmd(combine_gprofiler_str)
@@ -518,7 +518,7 @@ def pyminer_analysis(infile=None,
 
     if os.path.isdir(temp+"/sample_clustering_and_summary/significance/"):
         process_dir(high_marker_out_dir)
-        get_markers_call = 'get_high_marker_genes.py -m '+group_means_file
+        get_markers_call = 'python3 -m pyminer.get_high_marker_genes -m '+group_means_file
         get_markers_call += ' -sig '+aov_file
         get_markers_call += ' -out '+high_marker_out_dir
         get_markers_call += ' -annotation_dict '+annotation_dict
@@ -527,7 +527,7 @@ def pyminer_analysis(infile=None,
 
         plot_subset_file = os.path.join(high_marker_out_dir+"subset_input.txt")
         sorted_markers = os.path.join(high_marker_out_dir,"best_sorted_markers.tsv")
-        marker_plot_call = "plot_gene_subset.py -i "+infile
+        marker_plot_call = "python3 -m pyminer.plot_gene_subset -i "+infile
         marker_plot_call += " -heatmap "
         marker_plot_call += " -ids "+id_file_name
         marker_plot_call += " -o "+high_marker_out_dir
@@ -542,10 +542,10 @@ def pyminer_analysis(infile=None,
                 sys.exit('for hdf5, we need the ID_list argument')
             else:
                 marker_plot_call += ' -columns '+columns
-        marker_plot_call_1 = marker_plot_call+" -noi "+ plot_subset_file
+        #marker_plot_call_1 = marker_plot_call+" -noi "+ plot_subset_file
         marker_plot_call_2 = marker_plot_call+" -noi "+ sorted_markers + " -no_clust -heatmap_name top_sorted_markers"
-        cmd(marker_plot_call_1)
-        all_calls.append(marker_plot_call_1)
+        #cmd(marker_plot_call_1)
+        #all_calls.append(marker_plot_call_1)
         cmd(marker_plot_call_2)
         all_calls.append(marker_plot_call_2)
 
@@ -555,8 +555,9 @@ def pyminer_analysis(infile=None,
     #process_dir(temp+'/graphs')
     process_dir(temp+'/pos_cor_graphs')
 
-    full_network_plot_call = "plot_network.py -id_list "+temp+'/ID_list.txt'+" -adj "+out_adj+" -plot_all  -out_dir "+temp+"/graphs"
-    pos_cor_network_call = "plot_network.py -id_list "+temp+'/ID_list.txt'+" -adj "+out_adj_pos+" -plot_all  -out_dir "+temp+"/pos_cor_graphs"
+    full_network_plot_call = "python3 -m pyminer.plot_network -id_list "+temp + \
+        '/ID_list.txt'+" -adj "+out_adj+" -plot_all  -out_dir "+temp+"/graphs"
+    pos_cor_network_call = "python3 -m pyminer.plot_network -id_list "+temp+'/ID_list.txt'+" -adj "+out_adj_pos+" -plot_all  -out_dir "+temp+"/pos_cor_graphs"
 
     if os.path.isfile(temp+"/sample_clustering_and_summary/k_group_enrichment.tsv"):
         full_network_plot_call+=" -node_attrs "+temp+"/sample_clustering_and_summary/k_group_enrichment.tsv -range "+str(zscore_cutoff)+",-"+str(zscore_cutoff)
@@ -598,7 +599,7 @@ def pyminer_analysis(infile=None,
         
 
         ## now run the call
-        noi_call = "get_degrees_of_separation_from_adjacency_list.py -adj "+out_adj_pos
+        noi_call = "python3 -m pyminer.get_degrees_of_separation_from_adjacency_list -adj "+out_adj_pos
         noi_call += " -ids "+temp+"/ID_list.txt"
         noi_call += " -noi "+','.join(final_noi_list)
         noi_call += " -out_dir "+temp+'/genes_of_interest'
@@ -608,7 +609,7 @@ def pyminer_analysis(infile=None,
         
 
         ## if the plots file is there plot the color over the positions
-        plot_subset_call = 'plot_gene_subset.py -i '+infile
+        plot_subset_call = 'python3 -m pyminer.plot_gene_subset -i '+infile
         plot_subset_call += " -out_dir "+temp+'/genes_of_interest'
         plot_subset_call += " -noi "+pass_through_arg
         plot_subset_call += " -id_list "+temp+'/ID_list.txt'
@@ -626,7 +627,7 @@ def pyminer_analysis(infile=None,
     #################################################
     if not no_cell_signals:
         process_dir(temp+'/autocrine_paracrine_signaling/')
-        cell_signals_call = "cell_signals.py -i "+temp+'/sample_clustering_and_summary/significance/significant_and_enriched_boolean_table.tsv '
+        cell_signals_call = "python3 -m pyminer.cell_signals -i "+temp+'/sample_clustering_and_summary/significance/significant_and_enriched_boolean_table.tsv '
         cell_signals_call += ' -o '+temp+'/autocrine_paracrine_signaling/'
         cell_signals_call += ' -species '+species
         if string_db_dir[-1]!='/':
@@ -635,7 +636,7 @@ def pyminer_analysis(infile=None,
         all_calls.append(cell_signals_call)
         cmd(cell_signals_call)
         if os.path.isfile(temp+'/autocrine_paracrine_signaling/all_cell_type_specific_interactions_gprofiler.tsv'):
-            combine_gprofiler2_str="combine_gprofiler_results.py -i "+temp+'/autocrine_paracrine_signaling/all_cell_type_specific_interactions_gprofiler.tsv'
+            combine_gprofiler2_str="python3 -m pyminer.combine_gprofiler_results -i "+temp+'/autocrine_paracrine_signaling/all_cell_type_specific_interactions_gprofiler.tsv'
             combine_gprofiler2_str+=' -o '+temp+'/autocrine_paracrine_signaling/'
             all_calls.append(combine_gprofiler2_str)
             cmd(combine_gprofiler2_str)
@@ -643,14 +644,14 @@ def pyminer_analysis(infile=None,
 
     #################################################
     if os.path.isfile(temp+'/pos_cor_graphs/communities.txt'):
-        community_analysis = "pyminer_analyze_communities.py -community_table "+temp+"/pos_cor_graphs/communities.txt -out_dir "+temp+"/pos_cor_graphs/community_analysis -z_score_table "+temp+"/sample_clustering_and_summary/k_group_enrichment.tsv -species "+species
+        community_analysis = "python3 -m pyminer.pyminer_analyze_communities -community_table "+temp+"/pos_cor_graphs/communities.txt -out_dir "+temp+"/pos_cor_graphs/community_analysis -z_score_table "+temp+"/sample_clustering_and_summary/k_group_enrichment.tsv -species "+species
         community_analysis += ' -ad '+temp+'/annotations.pkl'
         cmd(community_analysis)
 
 
     ################################################
     ## summarize the results via the website generator
-    website_call = "make_website.py -i "+temp_dir
+    website_call = "python3 -m pyminer.make_website -i "+temp_dir
     all_calls.append(website_call)
 
 
