@@ -340,7 +340,10 @@ def do_ray_log_odds(a, b=None, threads = None, verbose = False, shuffle = False)
     process_mem = 6#Gigs
     total_mem = 24
     print("getting delta_log_odds")
-    ray.init(memory = int(total_mem*1000000000), object_store_memory=int(process_mem*1000000000))
+    try:
+        ray.init(memory = int(total_mem*1000000000), object_store_memory=int(process_mem*1000000000))
+    except:
+        ray.init()
     ray_c = ray.put(c)
     ray_jobs = []
     for thread in range(threads+1):
